import argparse
import pickle
import sys
import os

settings_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                             "settings")

settings_file_name = ''
broker_name = ''


def run():
    # entrypoint
    pass


class Settings:
    token = ''
    api_key = ''
    api_secret = ''
    position_size = 0
    last_order_id = ''

start_settings = Settings()

def file_create_input(check_exist=True, start=False):
    setting_path = os.path.join(settings_path, settings_file_name)
    if check_exist and os.path.exists(setting_path):
        settings = pickle.load(open(setting_path, 'rb'))
        check_settings(settings.token, settings.api_key,
                       settings.api_secret, settings.position_size, start=start)
    else:
        token = input(
            "Submit your Strategytoken (found in your member area): ")
        api_key = input("Submit your " + broker_name + " API Key (ID): ")
        api_secret = input("Submit your " + broker_name + " API Secret: ")
        pos_size = input("Choose your Position Size per trade in %: ")
        pos_size = check_pos_size(pos_size)
        make_pickle_file(settings_path, settings_file_name,
                         token, api_key, api_secret, pos_size)
        check_settings(token, api_key, api_secret, pos_size, start=start)


def check_settings(token, api_key, api_secret, position_size, start=False):
    if start:
        start_settings.token = token
        start_settings.api_key = api_key
        start_settings.api_secret = api_secret
        start_settings.position_size = position_size
    else:
        while True:
            qr = input("Start Algo Trader Bot with"
                    "\n\tBroker: " + broker_name +
                    "\n\tToken: {}..."
                    "\n\tAPI ID: {}"
                    "\n\tAPI Secret: ...{}"
                    "\n\tPosition Size: {} %\n\n"
                    "Do you want to continue? ['y' for yes, 'n' for no, 'c' for"
                    " change settings] "
                    .format(token[:4], api_key, api_secret[-4:], position_size))
            if qr == '' or qr[0].lower() not in ['y', 'n', 'c']:
                print("Please answer with 'y' for yes, 'n' for no or 'c' for "
                    "change settings (you don't have to write '').")
            elif qr[0].lower() == 'c':
                return file_create_input(check_exist=False)
            else:
                answer = qr[0].lower()
                if answer == 'y':
                    start_settings.token = token
                    start_settings.api_key = api_key
                    start_settings.api_secret = api_secret
                    start_settings.position_size = position_size
                    return
                else:
                    print("Shutting down...")
                    sys.exit()


def check_pos_size(pos_size, max_position_size=10):
    try:
        float(pos_size)
    except ValueError:
        new_size = input("Position Size is not a number,"
                         " please type it in again: ")
        return check_pos_size(new_size, max_position_size=max_position_size)
    if float(pos_size) > max_position_size:
        new_size = input("Position Size is over " + str(max_position_size) +
                         " %, please type in a lower size: ")
        return check_pos_size(new_size, max_position_size=max_position_size)
    return pos_size


def make_pickle_file(filepath, filename, token, api_key, api_secret, position_size):
    settings = Settings()
    settings.token = token
    settings.api_key = api_key
    settings.api_secret = api_secret
    settings.position_size = position_size
    pickle.dump(settings, file=open(os.path.join(filepath, filename), 'wb'))


parser = argparse.ArgumentParser()
parser.add_argument(
    'run', help='You have to say algotrader to \'run\' + symbol to trade and broker to trade on.')
parser.add_argument(
    'symbol', help='Define on which Symbol to trade. E.g.: BTCUSD for Bitcoin vs Dollar'
)
parser.add_argument(
    'broker', help='Define on which broker to run the strategy')
parser.add_argument('start', nargs='?', default='')

args = parser.parse_args()

if args.run in ['run', 'Run', 'RUN']:
    symbol = ''
    if args.symbol in ['BTCUSD', 'Bitcoin', 'bitcoin', 'XBTUSD', 'BTC', 'btc']:
        symbol = 'XBTUSD'
    elif args.symbol in ['ETHUSD', 'Ehtereum', 'ETH', 'eth']:
        symbol = 'ETHUSD'
    else:
        print('This symbol can\'t be traded with the the Algo Trader Bot.')
        sys.exit()

    testnet = True
    broker = ''
    if args.broker in ['bitmex_testnet', 'bitmextestnet', 'Bitmextestnet', 'Bitmex-Testnet', 'bitmex-testnet']:
        broker = 'bitmex_testnet'
        broker_name = 'Bitmex Testnet'
    elif args.broker in ['bitmex', 'bitmex_live', 'Bitmex', 'Bitmex-Live', 'bitmex-live']:
        testnet = False
        broker = 'bitmex'
        broker_name = 'Bitmex Live'
    else:
        print('This broker is not supported.')
        sys.exit()

    settings_file_name = '{}_lotus_{}_settings.pickle'.format(broker, symbol)

    if args.start == 'start':
        file_create_input(start=True)
    else:
        file_create_input()

    from algo_trader.strategies import Lotus
    from algo_trader.clients import BitmexClient

    client = BitmexClient(symbol=symbol, api_key=start_settings.api_key,
                          api_secret=start_settings.api_secret, test=testnet)
    lotus = Lotus(client, token=start_settings.token, pos_size=start_settings.position_size,
                  testnet=testnet, settings_file_name=settings_file_name)
    lotus.run()
else:
    print("Sorry, your entries were not matched with a valid symbol and broker. You need to input e.g. 'algotrader run btcusd bitmex_testnet' to run the Algo Trader Bot.")



import sys
import random
import datetime, time
from dateutil import parser
from algo_trader.clients import BitmexOrder
import requests
import os

class Lotus:
    def __init__(self, client, token, pos_size, testnet, settings_file_name):
        self.client = client
        self._token = token
        self._pos_size = pos_size
        self._testnet = testnet
        self._settings_path = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir, 'settings', settings_file_name))
        self._last_timestamp = datetime.datetime.utcnow()
        self._last_entry = 0
        self._last_sl = 0

        if not self.client.is_connected:
            print(
                "Invalid API ID or API Secret, please restart and provide the right keys", flush=True)
            # empty the inputs
            sys.exit()
        
        if testnet:
            from algo_trader.settings import TESTNET_LOTUS_LINK
            self._api_endpoint = TESTNET_LOTUS_LINK
        else:
            from algo_trader.settings import LIVE_LOTUS_LINK
            self._api_endpoint = LIVE_LOTUS_LINK

    def get_api_signal(self):
        auth = { 'token': self._token }
        r = requests.get(self._api_endpoint, params=auth)
        json = r.json()
        timestamp = parser.parse(json['timestamp'])
        if self._last_timestamp != timestamp:
            self._last_timestamp = timestamp
            if self._last_entry != json['entry'] and self._last_sl != json['stoploss']:
                self._last_entry = json['entry']
                self._last_sl = json['stoploss']
                return json['signal'], json['entry'], json['stoploss']
            else:
                return False, False, False
        else:
            return False, False, False

    def rand_sec(self, start=1, end=10):
        return random.randrange(start, end)

    def check_modify(self, sl):
        res = self.order.modifiy_stop(sl)
        if type(res) is str:
            print('Modify order Error:', res, flush=True)
        else:
            print('Modifying order to new stoploss:', sl, flush=True)
            self.order.SL = sl

    def process_signal(self, start=False):
        newsignal, entry, stoploss = self.get_api_signal()
        if start:
            self.order.check_open_bot_order(entry, stoploss)
            time.sleep(3)
        if newsignal:
            if newsignal in [-1, 1]:  # signal has open position
                if newsignal == 1:
                    if self.order.Qty > 0 or self.client.open_contracts > 0:
                        self.check_modify(stoploss)
                    elif self.client.open_contracts < 0:
                        print('Closing order.', flush=True)
                        close = self.order.close(qty=-self.order.Qty)
                        if close:
                            print('Closed open Bot old short position, because now the Bot\'s Position is long.', flush=True)
                        else:
                            print('Error closing order.', flush=True)
                    else:
                        cp = self.client.last_current_price
                        if cp < entry:
                            sl_dist = abs(stoploss-cp)
                            if sl_dist > 0.15 * sl_dist:
                                time.sleep(2)
                                contracts = self.order.calc_pos_size(sl_dist)
                                print("Open market stop order long into open Signal. Contracts {} Stoploss {}".format(contracts, stoploss), flush=True)
                                self.order.bracket_market_order(contracts, stoploss)
                else:
                    if self.order.Qty < 0 or self.client.open_contracts < 0:
                        self.check_modify(stoploss)
                    elif self.client.open_contracts > 0:
                        print('Closing order.', flush=True)
                        close = self.order.close(qty=-self.order.Qty)
                        if close:
                            print('Closed open Bot old long position, because now the Bot\'s Position is short.', flush=True)
                        else:
                            print('Error closing order.', flush=True)
                    else:
                        cp = self.client.last_current_price
                        if cp > entry:
                            sl_dist = abs(entry-cp)
                            if sl_dist > 0.15 * sl_dist:
                                time.sleep(2)
                                contracts = self.order.calc_pos_size(sl_dist)
                                print("Open market stop order short into open Signal. Contracts {} Stoploss {}".format(contracts, stoploss), flush=True)
                                self.order.bracket_market_order(-contracts, stoploss)
            elif newsignal in [-2, 2]:  # signal has pending order
                if self.order.WaitStop:
                    print("Cancel old order first.")
                    self.order.stoporder_cancel()  # cancel old order first
                    time.sleep(3)
                contracts = self.order.calc_pos_size(abs(entry - stoploss))
                if self.order.Open:
                    print("New signal is bracket stop long, but one bot order is open... closing open order.", flush=True)
                    self.order.close()
                    time.sleep(3)
                if newsignal == 2:
                    if self.order.Qty < 0:
                        print("Canceling stop order in other direction.", flush=True)
                        self.order.stoporder_cancel()
                        time.sleep(3)
                    print("Open bracket stop order long. Contracts {} Entry {} Stoploss {}".format(contracts, entry, stoploss), flush=True)
                    self.order.bracket_stop_order(contracts, entry, stoploss)
                else:
                    if self.order.Qty > 0:
                        print("Canceling stop order in other direction.", flush=True)
                        self.order.stoporder_cancel()
                        time.sleep(3)
                    print("Open bracket stop order short. Contracts {} Entry {} Stoploss {}".format(abs(contracts), entry, stoploss), flush=True)
                    self.order.bracket_stop_order(-contracts, entry, stoploss)
            else:
                if self.order.Open:
                    self.order.close()

    def run(self):
        # Main loop
        self.order = BitmexOrder(self.client, self._settings_path, position_size=self._pos_size)
        self.process_signal(start=True)
        while True:
            time.sleep(10)
            self.order.manage_entries()
            if datetime.datetime.now().minute % 30 == 0:
                time.sleep(19 + self.rand_sec(start=3))
                self.process_signal()
                time.sleep(40)



import random
import json
import math
import bitmex
import time
import datetime
import uuid
import pickle

from bravado.exception import HTTPUnauthorized, HTTPBadRequest, HTTPNotFound

class BitmexClient:
    def __init__(self, symbol='XBTUSD', api_key=None, api_secret=None, test=True):
        self.client = bitmex.bitmex(api_key=api_key, api_secret=api_secret, test=test)
        self.symbol = symbol
        self._last_currentprice = 0

    def __call__(self):
        return self.client

    def getHistory(self, symbol='XBTUSD', binSize='5m', count=15):
        try:
            return self.client.Trade.Trade_getBucketed(symbol=symbol,
                                                    binSize=binSize,
                                                    count=count,
                                                    reverse=True,
                                                    ).result()
        except HTTPBadRequest as e:
            if 'expired' in str(e):
                now = datetime.datetime.utcnow()
                print('Get history expired error. Time:', now, ' Server Time:', now.strftime("%m/%d/%Y, %H:%M:%S"), 'Returning last price.', flush=True)
            else:
                print('Error getting history', e, flush=True)    

    def getXBTUSDPosition(self, prop):
        return self.client.Position.Position_get(
            filter=json.dumps({'symbol': self.symbol})).result()[0][0][str(prop)]

    def getWallet(self, prop):
        return self.client.User.User_getWallet().result()[0][str(prop)]

    def getPosition(self, property):
        return self.client.Position.Position_get(
            filter=json.dumps({'symbol': self.symbol})).result()[0][0][property]

    @property
    def is_connected(self):  # [test]
        if not self.acc_balance:
            return False
        else:
            return True

    @property
    def acc_balance(self):
        try:
            return self.getWallet('amount')
        except HTTPUnauthorized:
            return False

    @property
    def unrealisedPnl(self):
        return self.getPosition('unrealisedPnl')

    @property
    def open_contracts(self):
        return self.getPosition('currentQty')

    @property
    def current_price_position(self):
        return self.getPosition('lastPrice')

    @property
    def last_current_price(self):
        if self._last_currentprice:
            return self._last_currentprice
        else:
            time.sleep(2)
            return self.current_price

    @property
    def current_price(self):
        cp = 0
        try:
            cp = self.client.Trade.Trade_get(symbol='XBTUSD',
                                             count=2,
                                             reverse=True,
                                             ).result()[0][0]['price']
            self._last_currentprice = cp
            return cp
        except (IndexError, HTTPBadRequest) as e:
            if 'expired' in str(e):
                now = datetime.datetime.utcnow()
                print('Current price expired error. Time:', now, ' Server Time:', now.strftime("%m/%d/%Y, %H:%M:%S"), 'Returning last price.', flush=True)
            return self._last_currentprice


class BitmexOrder:
    def __init__(self, client, settings_path, position_size=1, symbol='XBTUSD', magic='algo-trader_'):
        self._position_size = position_size  # in %
        self.client = client
        self.Magic = magic
        self.BotID = self.Magic + symbol + '_'
        self.symbol = symbol
        self._settings_path = settings_path

        self.Entry = 0 
        self.WaitStop = False
        self.Open = False
        self.Qty = 0

        self.StopID = ''
        self.StoplossID = ''
        self.TakeprofitID = ''

        self.SL = 0
        self.TP = 0

    def check_open_bot_order(self, entry, sl):
        settings = pickle.load(open(self._settings_path, 'rb'))
        self.StoplossID = settings.last_order_id
        if self.StoplossID:
            try:
                res = self.client.client.Order.Order_getOrders(filter=json.dumps({"symbol": "XBTUSD", "open": True})).result()
                if self.StoplossID in [order['clOrdID'] for order in res[0]]:
                    # bot order is open and sl might be modified
                    self.modifiy_stop(sl)
                    self.Entry = entry
                    self.save_open_state(True)
            except HTTPBadRequest as e:
                print("Error getting orders.", e)

    def save_open_id(self, id):
        self.StoplossID = id
        settings = pickle.load(open(self._settings_path, 'rb'))
        settings.last_order_id = self.StoplossID
        pickle.dump(settings, file=open(self._settings_path, 'wb'))

    def manage_entries(self):
        cp = self.client.current_price
        if self.WaitStop:
            time.sleep(2)
            if self.Qty > 0 and cp >= self.Entry and self.client.open_contracts != 0:
                self.save_pending_state(False)
                self.save_open_state(True)
                print("Long Position has been opened, amending Stoploss to", self.SL, flush=True)
                self.stoploss_order()
            elif self.Qty < 0 and cp <= self.Entry and self.client.open_contracts != 0:
                self.save_pending_state(False)
                self.save_open_state(True)
                print("Short Position has been opened, amending Stoploss to", self.SL, flush=True)
                self.stoploss_order()
        elif self.Open:
            if self.Qty > 0 and cp < self.SL and self.client.open_contracts == 0:
                self.save_open_state(False)
                self.save_open_contracts(0)
                print("Long Position closed. {} Contracts from {} to {}. PnL in Points: {}".format(self.Qty, self.Entry, self.SL, self.bitmex_decimals(cp-self.Entry)), flush=True)
            elif self.Qty < 0 and cp > self.SL and self.client.open_contracts == 0:
                self.save_open_state(False)
                self.save_open_contracts(0)
                print("Short Position closed. {} Contracts from {} to {}. PnL in Points: {}".format(-self.Qty, self.Entry, self.SL, self.bitmex_decimals(cp-self.Entry)), flush=True)
            else:
                self.save_open_state(False)

    def generate_id(self, _type='stop'):
        return self.BotID + str(uuid.uuid4().fields[-1])[:5] + '_' + _type

    def save_open_contracts(self, orderQty):
        self.Qty = orderQty

    def save_pending_state(self, state):
        self.WaitStop = state

    def save_open_state(self, state):
        self.Open = state

    def bitmex_decimals(self, price):
        return round(price * 2) / 2

    def order_id():
        return str(random.randint(100000, 999999))

    def calc_pos_size(self, sl_distance):
        acc_balance = self.client.acc_balance
        cp = self.client.last_current_price
        riskvalue = float(acc_balance) / 100000000 * float(cp) * (float(self._position_size)/100)
        pos_size = int(round((math.floor(riskvalue / sl_distance * 5000) / 100)) * 100)
        if pos_size < 100:
            return 100
        else:
            return pos_size

    def stoploss_order(self, execPrice='LastPrice'):
        try:
            self.StoplossID = self.generate_id(_type='SL')
            self.save_open_id(self.StoplossID)
            return self.client.client.Order.Order_new(
                symbol=self.symbol, ordType='Stop', clOrdID=self.StoplossID, orderQty=-self.Qty,
                stopPx=self.bitmex_decimals(self.SL), execInst=execPrice
            ).result()
        except HTTPBadRequest as e:
            print("Error placing stoploss order.", e, flush=True)

    def order_cancel(self, orderId):
        try:
            order_res = self.client.client.Order.Order_cancel(
                clOrdID=orderId
            ).result()
            self.WaitStop = False
            return order_res
        except (HTTPBadRequest, HTTPNotFound) as e:
            print("Error canceling order.", e, flush=True)

    def stoploss_cancel(self):
        self.order_cancel(self.StoplossID)

    def stoporder_cancel(self):
        self.order_cancel(self.StopID)

    def bracket_stop_order(self, orderQty, entry, sl, tp=0):
        try:
            self.StopID = self.generate_id()
            order_res = self.client.client.Order.Order_new(
                symbol=self.symbol, ordType='Stop', clOrdID=self.StopID, orderQty=orderQty,
                stopPx=self.bitmex_decimals(entry), execInst='LastPrice'
            ).result()
            self.save_open_contracts(orderQty)
            self.save_pending_state(True)
            self.SL = sl
            self.TP = tp
            self.Entry = entry
            return order_res
        except HTTPBadRequest as e:
            print("Error placing bracket stop order.", e, flush=True)

    def bracket_market_order(self, orderQty, sl, tp=0):
        try:
            entry = self.market_order(orderQty)
            self.SL = sl
            self.TP = tp
            time.sleep(5)
            stoploss = self.stoploss_order()
            self.save_open_contracts(orderQty)
            self.save_open_state(True)
            return entry, stoploss
        except HTTPBadRequest as e:
            print("Error placing bracket market order.", e, flush=True)

    def market_order(self, orderQty):
        try:
            order_res = self.client.client.Order.Order_new(
                symbol=self.symbol, ordType='Market', orderQty=orderQty
            ).result()
            self.save_open_contracts(orderQty)
            self.save_open_state(True)
            return order_res
        except HTTPBadRequest as e:
            print("Error placing market order.", e, flush=True)

    def modifiy_stop(self, newPrice):
        if self.WaitStop:
            try:
                res = self.client.client.Order.Order_amend(
                    orderID=self.StoplossID, stopPx=self.bitmex_decimals(
                        newPrice)
                ).result()
                return res
            except HTTPBadRequest as e:
                if 'Invalid amend' in str(e):
                    return 'noValChanged'
                elif 'Invalid origClOrdID' in str(e):
                    return 'invalidID'
                else:
                    return str(e)
        else:
            return False

    def close(self, qty=0):
        if not qty:
            qty = -self.Qty
        if self.Open:
            try:
                return self.client.client.Order.Order_new(
                    symbol=self.symbol, ordType='Market', execInst='Close', orderQty=qty
                ).result()
            except HTTPBadRequest as e:
                print("Error closing order.", e, flush=True)
        else:
            return False



import setuptools
from os.path import dirname, join

here = dirname(__file__)


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="algo-trader",
    version="1.0.2",
    author="Niclas Hummel",
    author_email="info@algoinvest.online",
    description="Trade execution engine to process API data and transmit"
    " orders to Bitmex and other brokers.",
    long_description=open(join(here, 'README.md')).read(),
    long_description_content_type='text/markdown',
    url="https://github.com/dignitas123/algo_trader",
    install_requires=['bitmex'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',    
    entry_points={
        'console_scripts': [
            'algotrader=algo_trader.startbot:run',
        ],
    }
)
